package com.docus.server.common.netty.server.handler;

import com.docus.core.util.StringUtils;
import com.docus.core.util.json.JSON;
import com.docus.server.common.MsgConstants;
import com.docus.server.common.netty.CommMsg;
import com.docus.server.common.netty.server.ChannelRepository;
import com.docus.server.enums.OnlineStateEnum;
import com.fasterxml.jackson.core.type.TypeReference;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.group.ChannelGroup;
import io.netty.channel.group.DefaultChannelGroup;
import io.netty.util.concurrent.GlobalEventExecutor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.net.InetSocketAddress;

/**
 * 业务消息处理
 */
@Slf4j
@ChannelHandler.Sharable
@Component
public class NettyBusinessHandler extends SimpleChannelInboundHandler<ByteBuf> {

    private static final ChannelGroup DEFAULT_CHANNEL_GROUP = new DefaultChannelGroup(GlobalEventExecutor.INSTANCE);

    @Resource
    private ChannelRepository repository;

    @Override
    protected void channelRead0(ChannelHandlerContext channelHandlerContext, ByteBuf msg) throws Exception {
        ByteBuf buf = (ByteBuf) msg;
        //创建目标大小的数组
        byte[] barray = new byte[buf.readableBytes()];
        //把数据从bytebuf转移到byte[]
        buf.getBytes(0, barray);
        //将byte[]转成字符串用于打印
        String message = new String(barray);

        //空消息不处理
        if (!StringUtils.hasText(message)) {
            return;
        }

        CommMsg<TerminatorContent> commMsg = JSON.fromJSONWithGeneric(message, new TypeReference<CommMsg<TerminatorContent>>() {
        });

        String messageType = commMsg.getMessageType();
        String messageTime = commMsg.getMessageTime();
        TerminatorContent messageContent = commMsg.getContent();

        if (messageType.equals(MsgConstants.TERMINATOR_RESTART)) {
            log.info("接受到终端重启命令，内容{}", messageContent);
        }

        if (messageType.equals(MsgConstants.COLLECTOR_RESTART)) {
            log.info("收到采集器重启命令，内容{}", messageContent);
        }

        if (messageType.equals(MsgConstants.VIRTUAL_RESTART)) {
            log.info("收到虚拟机重启命令，内容{}", messageContent);
        }

        if (messageType.equals(MsgConstants.UPDATE_COLLECTOR_FILE)) {
            log.info("收到更新采集器文件命令，内容{}", messageContent);
        }

        if (messageType.equals(MsgConstants.UPDATE_COLLECTOR_CONFIG)) {
            log.info("收到更新采集器配置命令，内容{}", messageContent);
        }

        if (messageType.equals(MsgConstants.SCH_DISTRIBUTE_TASKS)) {
            log.info("收到采集调度器下发任务命令，内容{}", messageContent);
        }
    }

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        ctx.fireChannelRegistered();

        InetSocketAddress ipSocket = (InetSocketAddress) ctx.channel().remoteAddress();
        String clientIp = ipSocket.getAddress().getHostAddress();
        log.info("【采集器-终端IP】:{}，连接上线,IP地址信息:{}", clientIp, clientIp);
        Channel channel = repository.get(clientIp);
        if (channel != null && channel.isOpen()) {
            channel.close();
        }

        repository.put(clientIp, ctx.channel(), OnlineStateEnum.OFFLINE);

        DEFAULT_CHANNEL_GROUP.add(ctx.channel());

        System.out.println(ctx.channel().remoteAddress() + " 上线," + "【采集器-终端】在线数量：" + DEFAULT_CHANNEL_GROUP.size());
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        Channel channel = ctx.channel();
        System.out.println(channel.remoteAddress() + " 下线," + "【采集器-终端】在线数量：" + DEFAULT_CHANNEL_GROUP.size());
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        cause.printStackTrace();
        Channel channel = ctx.channel();
        if (channel.isActive()) {
            ctx.close();
        }

        super.exceptionCaught(ctx, cause);
    }

}
